import math


class Planet:
    """
    Данный класс содержит родительский класс планеты и дочерние каждой планеты
    Для создания планеты необходимо пространство
    """
    def __init__(self, name, space):
        self.name = name
        self.space = space

    def getDeg(self):
        ephem_planet = getattr(__import__('ephem'), self.name)()
        ephem_planet.compute(self.space.date)
        return math.degrees(ephem_planet.ra)

    def getSegment(self):
        """
        Получить сегмент для планеты
        """
        for segment in self.space.segments:
            if segment.getDegFrom() <= self.getDeg() < segment.getDegTo():
                return segment


class Sun(Planet):

    id = 1
    name = 'Sun'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Moon(Planet):

    id = 2
    name = 'Moon'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Mercury(Planet):

    id = 3
    name = 'Mercury'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)

class Venus(Planet):

    id = 4
    name = 'Venus'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Mars(Planet):

    id = 5
    name = 'Mars'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Jupiter(Planet):

    id = 6
    name = 'Jupiter'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Saturn(Planet):

    id = 7
    name = 'Saturn'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Uranus(Planet):

    id = 8
    name = 'Uranus'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Neptune(Planet):

    id = 9
    name = 'Neptune'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)


class Pluto(Planet):

    id = 10
    name = 'Pluto'

    def __init__(self, space):
        Planet.__init__(self, self.name, space)





from dispositor.space import Space


class Chain:
    """
    Этот класс возвращает данные только для цепочки диспозиторов (части цепочки, содержимое части (планеты с орбитами))
    Предположительно изначально будет расчет вестись без учета орбит. Поэтому пока данный класс не реализуется.
    """
    def __init__(self, space):
        self.space = space


class Segment:
    def __init__(self, number, id, name, rus_name, owner_planet, deg_from, deg_to):
        self.number = number
        self.id = id
        self.deg_from = deg_from
        self.deg_to = deg_to
        self.name = name
        self.rus_name = rus_name
        self.owner_planet = owner_planet

    def getDegNumber(self):
        return self.number

    def getDegFrom(self):
        return self.deg_from

    def getDegTo(self):
        return self.deg_to




from setuptools import setup

setup(name='dispositor',
      version='1.2',
      description='Library for calculating the zodiac sign',
      packages=['dispositor'],
      author_email='astro.slfd@gmail.com',
      zip_safe=False)


import dispositor.planet as planet
from dispositor.segment import Segment


class Space:
    """
    Класс пространства.
    Для создания необходима дата и сегменты (деление пространства)
    Обязанность: Создание сегментов и планет.
    """
    def __init__(self, date, segmentData):
        self.date = date
        self.segmentData = segmentData
        self.deg_count = 360
        self.segments = self.createSegments()
        self.planets = self.createPlanets()

    def createPlanets(self):
        """
        Уникальные значения планет из списка сегментов
        """
        planets = []
        for owner_planet in [segment.owner_planet for segment in self.segments]:
            if owner_planet.__name__ not in [planet_class.__class__.__name__ for planet_class in planets]:
                planets.append(owner_planet(self))
        return planets

    def createSegments(self):
        """
        Фабрика сегментов
        """
        segments = []
        for i, segment_data in enumerate(self.segmentData):
            deg_in_segment = self.getDegCount() / self.getSegmentCount()
            deg_from = i * deg_in_segment
            deg_to = deg_from + deg_in_segment
            segments.append(Segment(
                i+1,
                segment_data['id'],
                segment_data['name'],
                segment_data['rus_name'],
                segment_data['owner_planet'],
                deg_from, deg_to
            ))
        return segments

    def getSegmentCount(self):
        """
        Получить количество сегментов
        """
        return len(self.segmentData)

    def getDegCount(self):
        """
        Получить количество градусов в окружности (пространстве)
        """
        return self.deg_count

    def getPlanets(self):
        """
        Получить список планет
        """
        return self.planets

    def getSegments(self):
        """
        Получить список сегментов
        """
        return self.segments


def classicSegmentData():
    """
    Классическое расположение сегментов и планет
    """
    return [
        {'id': 1, 'name': 'Aries', 'rus_name': 'Овен', 'owner_planet': planet.Mars},
        {'id': 2, 'name': 'Taurus', 'rus_name': 'Телец', 'owner_planet': planet.Venus},
        {'id': 3, 'name': 'Gemini', 'rus_name': 'Близнецы', 'owner_planet': planet.Mercury},
        {'id': 4, 'name': 'Cancer', 'rus_name': 'Рак', 'owner_planet': planet.Moon},
        {'id': 5, 'name': 'Leo', 'rus_name': 'Лев', 'owner_planet': planet.Sun},
        {'id': 6, 'name': 'Virgo', 'rus_name': 'Дева', 'owner_planet': planet.Mercury},
        {'id': 7, 'name': 'Libra', 'rus_name': 'Весы', 'owner_planet': planet.Venus},
        {'id': 8, 'name': 'Scorpio', 'rus_name': 'Скорпион', 'owner_planet': planet.Pluto},
        {'id': 9, 'name': 'Sagittarius', 'rus_name': 'Стрелец', 'owner_planet': planet.Jupiter},
        {'id': 10, 'name': 'Capricorn', 'rus_name': 'Козерог', 'owner_planet': planet.Saturn},
        {'id': 11, 'name': 'Aquarius', 'rus_name': 'Водолей', 'owner_planet': planet.Uranus},
        {'id': 12, 'name': 'Pisces', 'rus_name': 'Рыбы', 'owner_planet': planet.Neptune},
    ]
def septenerSegmentData():
    """
    Расположение сегментов и планет
    """
    return [
        {'id': 1, 'name': 'Aries', 'rus_name': 'Овен', 'owner_planet': planet.Mars},
        {'id': 2, 'name': 'Taurus', 'rus_name': 'Телец', 'owner_planet': planet.Venus},
        {'id': 3, 'name': 'Gemini', 'rus_name': 'Близнецы', 'owner_planet': planet.Mercury},
        {'id': 4, 'name': 'Cancer', 'rus_name': 'Рак', 'owner_planet': planet.Moon},
        {'id': 5, 'name': 'Leo', 'rus_name': 'Лев', 'owner_planet': planet.Sun},
        {'id': 6, 'name': 'Virgo', 'rus_name': 'Дева', 'owner_planet': planet.Mercury},
        {'id': 7, 'name': 'Libra', 'rus_name': 'Весы', 'owner_planet': planet.Venus},
        {'id': 8, 'name': 'Scorpio', 'rus_name': 'Скорпион', 'owner_planet': planet.Mars},
        {'id': 9, 'name': 'Sagittarius', 'rus_name': 'Стрелец', 'owner_planet': planet.Jupiter},
        {'id': 10, 'name': 'Capricorn', 'rus_name': 'Козерог', 'owner_planet': planet.Saturn},
        {'id': 11, 'name': 'Aquarius', 'rus_name': 'Водолей', 'owner_planet': planet.Saturn},
        {'id': 12, 'name': 'Pisces', 'rus_name': 'Рыбы', 'owner_planet': planet.Jupiter},
    ]